'use strict'
var init = require('../helpers/init')
var sauce = require('./sauce')

module.exports = {
  before: function (browser, done) {
    init(browser, done)
  },
  '@sources': function () {
    return sources
  },
  'Use special functions receive/fallback - both are declared, sending data': function (browser) {
    browser.waitForElementVisible('#icon-panel', 10000)
    .testContracts('receiveAndFallback.sol', sources[0]['browser/receiveAndFallback.sol'], ['CheckSpecials'])
    .clickLaunchIcon('udapp')
    .selectContract('CheckSpecials')
    .createContract('')
    .clickInstance(0)
    .perform((done) => {
      browser.getAddressAtPosition(0, (address) => {
        browser.sendLowLevelTx(address, '0', '0xaa')
        .pause(1000)
        .journalLastChildIncludes('to:CheckSpecials.(fallback)')
        .journalLastChildIncludes('value:0 wei')
        .journalLastChildIncludes('data:0xaa')
        .waitForElementVisible(`#instance${address} label[id="deployAndRunLLTxError"]`)
        .assert.containsText(`#instance${address} label[id="deployAndRunLLTxError"]`, '')
        .perform(done)
      })
    })
  },
  'Use special functions receive/follback - both are declared - fallback should fail cause not payable, sending data and wei': function (browser) {
    // don't need to redeploy it, same contract
    browser.perform((done) => {
      browser.getAddressAtPosition(0, (address) => {
        browser.sendLowLevelTx(address, '10', '0xaa')
        .pause(1000)
        .journalLastChildIncludes('The called function should be payable if you send value')
        .perform(done)
      })
    })
  },
  'Use special functions receive/follback - fallback is not declared, sending data': function (browser) {
    browser.waitForElementVisible('#icon-panel', 10000)
      .testContracts('receiveOnly.sol', sources[0]['browser/receiveOnly.sol'], ['CheckSpecials'])
      .clickLaunchIcon('udapp')
      .selectContract('CheckSpecials')
      .createContract('')
      .clickInstance(1)
      .perform((done) => {
        browser.getAddressAtPosition(1, (address) => {
          browser.sendLowLevelTx(address, '0', '0xaa')
          .pause(1000)
          .waitForElementVisible(`#instance${address} label[id="deployAndRunLLTxError"]`)
          .assert.containsText(`#instance${address} label[id="deployAndRunLLTxError"]`, `'fallback' function is not defined`)
          .perform(done)
        })
      })
    .end()
  },
  'Use special functions receive/fallback - receive is not declared, fallback is payable': function (browser) {
    browser.waitForElementVisible('#icon-panel', 10000)
      .testContracts('fallbackOnlyPayable.sol', sources[0]['browser/fallbackOnlyPayable.sol'], ['CheckSpecials'])
      .clickLaunchIcon('udapp')
      .selectContract('CheckSpecials')
      .createContract('')
      .clickInstance(2)
      .perform((done) => {
        browser.getAddressAtPosition(1, (address) => {
          browser.sendLowLevelTx(address, '1', '')
          .pause(1000)
          .waitForElementVisible(`#instance${address} label[id="deployAndRunLLTxError"]`)
          .assert.containsText(`#instance${address} label[id="deployAndRunLLTxError"]`, '')
          .journalLastChildIncludes('to:CheckSpecials.(fallback)')
          .journalLastChildIncludes('value:1 wei')
          .perform(done)
        })
      })
    .end()
  },
  'Use special functions receive/follback - receive is not declared, fallback is payable, sending data and wei': function (browser) {
    // don't need to redeploy it, same contract
    perform((done) => {
      browser.getAddressAtPosition(1, (address) => {
        browser.sendLowLevelTx(address, '1', '0xaa')
        .pause(1000)
        .waitForElementVisible(`#instance${address} label[id="deployAndRunLLTxError"]`)
        .assert.containsText(`#instance${address} label[id="deployAndRunLLTxError"]`, '')
        .journalLastChildIncludes('to:CheckSpecials.(fallback)')
        .journalLastChildIncludes('value:1 wei')
        .journalLastChildIncludes('data:0xaa')
        .perform(done)
      })
    })
  },
  tearDown: sauce
}

var sources = [
  {
    'browser/receiveAndFallback.sol': {
      content: `
        contract CheckSpecials {
          receive() payable external{}
          fallback() external {}
        }
      `
    },
    'browser/receiveOnly.sol': {
      content: `
        contract CheckSpecials {
          receive() payable external {}
        }
      `
    },
    'browser/fallbackOnlyPayable.sol': {
      content: `
        contract CheckSpecials {
          fallback() payable external {}
        }
      `
    },
    'browser/fallbackOnly.sol': {
      content: `
        contract CheckSpecials {
          fallback() external {}
        }
      `
    },
    'browser/receiveAndFallbackBothPayable.sol': {
      content: `
        contract CheckSpecials {
          receive() payable external {}
          fallback() payable external {}
        }
      `
    }
  }
]
